@@ -4,11 +4,13 @@ Set the parameters for Covasim.
 
 import numpy as np
 import sciris as sc
+from .settings import options as cvo # For setting global options
+from . import misc as cvm
 
 __all__ = ['make_pars', 'reset_layer_pars', 'get_prognoses']
 
 
-def make_pars(set_prognoses=False, prog_by_age=True, **kwargs):
+def make_pars(set_prognoses=False, prog_by_age=True, version=None, **kwargs):
     '''
     Create the parameters for the simulation. Typically, this function is used
     internally rather than called by the user; e.g. typical use would be to do
@@ -17,8 +19,9 @@ def make_pars(set_prognoses=False, prog_by_age=True, **kwargs):
 
     Args:
         set_prognoses (bool): whether or not to create prognoses (else, added when the population is created)
-        prog_by_age (bool): whether or not to use age-based severity, mortality etc.
-        kwargs (dict): any additional kwargs are interpreted as parameter names
+        prog_by_age   (bool): whether or not to use age-based severity, mortality etc.
+        kwargs        (dict): any additional kwargs are interpreted as parameter names
+        version       (str):  if supplied, use parameters from this Covasim version
 
     Returns:
         pars (dict): the parameters of the simulation
@@ -27,8 +30,8 @@ def make_pars(set_prognoses=False, prog_by_age=True, **kwargs):
 
     # Population parameters
     pars['pop_size']     = 20e3     # Number of agents, i.e., people susceptible to SARS-CoV-2
-    pars['pop_infected'] = 10       # Number of initial infections
-    pars['pop_type']     = 'random' # What type of population data to use -- random (fastest), synthpops (best), hybrid (compromise), or clustered (not recommended)
+    pars['pop_infected'] = 20       # Number of initial infections
+    pars['pop_type']     = 'random' # What type of population data to use -- 'random' (fastest), 'synthpops' (best), 'hybrid' (compromise)
     pars['location']     = None     # What location to load data from -- default Seattle
 
     # Simulation parameters
@@ -36,7 +39,7 @@ def make_pars(set_prognoses=False, prog_by_age=True, **kwargs):
     pars['end_day']    = None         # End day of the simulation
     pars['n_days']     = 60           # Number of days to run, if end_day isn't specified
     pars['rand_seed']  = 1            # Random seed, if None, don't reset
-    pars['verbose']    = 1            # Whether or not to display information during the run -- options are 0 (silent), 1 (default), 2 (everything)
+    pars['verbose']    = cvo.verbose  # Whether or not to display information during the run -- options are 0 (silent), 1 (default), 2 (everything)
 
     # Rescaling parameters
     pars['pop_scale']         = 1    # Factor by which to scale the population -- e.g. pop_scale=10 with pop_size=100e3 means a population of 1 million
@@ -50,8 +53,8 @@ def make_pars(set_prognoses=False, prog_by_age=True, **kwargs):
     pars['dynam_layer'] = None  # Which layers are dynamic; set by reset_layer_pars() below
     pars['beta_layer']  = None  # Transmissibility per layer; set by reset_layer_pars() below
     pars['n_imports']   = 0     # Average daily number of imported cases (actual number is drawn from Poisson distribution)
-    pars['beta_dist']   = {'dist':'neg_binomial','par1':1.0, 'par2':0.45, 'step':0.01} # Distribution to draw individual level transmissibility; dispersion from https://www.researchsquare.com/article/rs-29548/v1
-    pars['viral_dist']  = {'frac_time':0.3, 'load_ratio':2, 'high_cap':4} # The time varying viral load (transmissibility); estimated from Lescure 2020, Lancet, https://doi.org/10.1016/S1473-3099(20)30200-0
+    pars['beta_dist']   = dict(dist='neg_binomial', par1=1.0, par2=0.45, step=0.01) # Distribution to draw individual level transmissibility; dispersion from https://www.researchsquare.com/article/rs-29548/v1
+    pars['viral_dist']  = dict(frac_time=0.3, load_ratio=2, high_cap=4) # The time varying viral load (transmissibility); estimated from Lescure 2020, Lancet, https://doi.org/10.1016/S1473-3099(20)30200-0
 
     # Efficacy of protection measures
     pars['asymp_factor'] = 1.0 # Multiply beta by this factor for asymptomatic cases; no statistically significant difference in transmissibility: https://www.sciencedirect.com/science/article/pii/S1201971220302502
@@ -61,17 +64,17 @@ def make_pars(set_prognoses=False, prog_by_age=True, **kwargs):
 
     # Duration parameters: time for disease progression
     pars['dur'] = {}
-    pars['dur']['exp2inf']  = {'dist':'lognormal_int', 'par1':4.6, 'par2':4.8} # Duration from exposed to infectious; see Lauer et al., https://www.ncbi.nlm.nih.gov/pmc/articles/PMC7081172/, subtracting inf2sim duration
-    pars['dur']['inf2sym']  = {'dist':'lognormal_int', 'par1':1.0, 'par2':0.9} # Duration from infectious to symptomatic; see Linton et al., https://doi.org/10.3390/jcm9020538
-    pars['dur']['sym2sev']  = {'dist':'lognormal_int', 'par1':6.6, 'par2':4.9} # Duration from symptomatic to severe symptoms; see Linton et al., https://doi.org/10.3390/jcm9020538
-    pars['dur']['sev2crit'] = {'dist':'lognormal_int', 'par1':3.0, 'par2':7.4} # Duration from severe symptoms to requiring ICU; see Wang et al., https://jamanetwork.com/journals/jama/fullarticle/2761044
+    pars['dur']['exp2inf']  = dict(dist='lognormal_int', par1=4.6, par2=4.8) # Duration from exposed to infectious; see Lauer et al., https://www.ncbi.nlm.nih.gov/pmc/articles/PMC7081172/, subtracting inf2sim duration
+    pars['dur']['inf2sym']  = dict(dist='lognormal_int', par1=1.0, par2=0.9) # Duration from infectious to symptomatic; see Linton et al., https://doi.org/10.3390/jcm9020538
+    pars['dur']['sym2sev']  = dict(dist='lognormal_int', par1=6.6, par2=4.9) # Duration from symptomatic to severe symptoms; see Linton et al., https://doi.org/10.3390/jcm9020538
+    pars['dur']['sev2crit'] = dict(dist='lognormal_int', par1=3.0, par2=7.4) # Duration from severe symptoms to requiring ICU; see Wang et al., https://jamanetwork.com/journals/jama/fullarticle/2761044
 
     # Duration parameters: time for disease recovery
-    pars['dur']['asym2rec'] = {'dist':'lognormal_int', 'par1':8.0,  'par2':2.0} # Duration for asymptomatic people to recover; see Wölfel et al., https://www.nature.com/articles/s41586-020-2196-x
-    pars['dur']['mild2rec'] = {'dist':'lognormal_int', 'par1':8.0,  'par2':2.0} # Duration for people with mild symptoms to recover; see Wölfel et al., https://www.nature.com/articles/s41586-020-2196-x
-    pars['dur']['sev2rec']  = {'dist':'lognormal_int', 'par1':14.0, 'par2':2.4} # Duration for people with severe symptoms to recover, 22.6 days total; see Verity et al., https://www.medrxiv.org/content/10.1101/2020.03.09.20033357v1.full.pdf
-    pars['dur']['crit2rec'] = {'dist':'lognormal_int', 'par1':14.0, 'par2':2.4} # Duration for people with critical symptoms to recover, 22.6 days total; see Verity et al., https://www.medrxiv.org/content/10.1101/2020.03.09.20033357v1.full.pdf
-    pars['dur']['crit2die'] = {'dist':'lognormal_int', 'par1':6.2,  'par2':1.7} # Duration from critical symptoms to death, 17.8 days total; see Verity et al., https://www.medrxiv.org/content/10.1101/2020.03.09.20033357v1.full.pdf
+    pars['dur']['asym2rec'] = dict(dist='lognormal_int', par1=8.0,  par2=2.0) # Duration for asymptomatic people to recover; see Wölfel et al., https://www.nature.com/articles/s41586-020-2196-x
+    pars['dur']['mild2rec'] = dict(dist='lognormal_int', par1=8.0,  par2=2.0) # Duration for people with mild symptoms to recover; see Wölfel et al., https://www.nature.com/articles/s41586-020-2196-x
+    pars['dur']['sev2rec']  = dict(dist='lognormal_int', par1=14.0, par2=2.4) # Duration for people with severe symptoms to recover, 22.6 days total; see Verity et al., https://www.medrxiv.org/content/10.1101/2020.03.09.20033357v1.full.pdf
+    pars['dur']['crit2rec'] = dict(dist='lognormal_int', par1=14.0, par2=2.4) # Duration for people with critical symptoms to recover, 22.6 days total; see Verity et al., https://www.medrxiv.org/content/10.1101/2020.03.09.20033357v1.full.pdf
+    pars['dur']['crit2die'] = dict(dist='lognormal_int', par1=6.2,  par2=1.7) # Duration from critical symptoms to death, 17.8 days total; see Verity et al., https://www.medrxiv.org/content/10.1101/2020.03.09.20033357v1.full.pdf
 
     # Severity parameters: probabilities of symptom progression
     pars['rel_symp_prob']   = 1.0  # Scale factor for proportion of symptomatic cases
@@ -97,7 +100,14 @@ def make_pars(set_prognoses=False, prog_by_age=True, **kwargs):
     pars.update(kwargs)
     reset_layer_pars(pars)
     if set_prognoses: # If not set here, gets set when the population is initialized
-        pars['prognoses'] = get_prognoses(pars['prog_by_age']) # Default to age-specific prognoses
+        pars['prognoses'] = get_prognoses(pars['prog_by_age'], version=version) # Default to age-specific prognoses
+
+    # If version is specified, load old parameters
+    if version is not None:
+        version_pars = cvm.get_version_pars(version)
+        for key in pars.keys(): # Only loop over keys that have been populated
+            if key in version_pars: # Only replace keys that exist in the old version
+                pars[key] = version_pars[key]
 
     return pars
 
@@ -108,17 +118,21 @@ layer_pars = ['beta_layer', 'contacts', 'dynam_layer', 'iso_factor', 'quar_facto
 
 def reset_layer_pars(pars, layer_keys=None, force=False):
     '''
-    Small helper function to set layer-specific parameters. If layer keys are not
-    provided, then set them based on the population type.
+    Helper function to set layer-specific parameters. If layer keys are not provided,
+    then set them based on the population type. This function is not usually called
+    directly by the user, although it can sometimes be used to fix layer key mismatches
+    (i.e. if the contact layers in the population do not match the parameters). More
+    commonly, however, mismatches need to be fixed explicitly.
 
     Args:
         pars (dict): the parameters dictionary
         layer_keys (list): the layer keys of the population, if available
-        force (bool): reset the pars even if they already exist
+        force (bool): reset the parameters even if they already exist
     '''
 
     # Specify defaults for random -- layer 'a' for 'all'
-    defaults_r = dict(
+    layer_defaults = {}
+    layer_defaults['random'] = dict(
         beta_layer  = dict(a=1.0), # Default beta
         contacts    = dict(a=20),  # Default number of contacts
         dynam_layer = dict(a=0),   # Do not use dynamic layers by default
@@ -126,27 +140,38 @@ def reset_layer_pars(pars, layer_keys=None, force=False):
         quar_factor = dict(a=0.3), # Assumed quarantine factor
     )
 
-    # Specify defaults for hybrid (and SynthPops) -- household, school, work, and community layers (h, s, w, c)
-    defaults_h = dict(
-        beta_layer  = dict(h=3.0, s=0.6, w=0.6, c=0.3), # Per-population beta weights; relative
+    # Specify defaults for hybrid -- household, school, work, and community layers (h, s, w, c)
+    layer_defaults['hybrid'] = dict(
+        beta_layer  = dict(h=3.0, s=0.6, w=0.6, c=0.3),  # Per-population beta weights; relative; in part based on Table S14 of https://science.sciencemag.org/content/sci/suppl/2020/04/28/science.abb8001.DC1/abb8001_Zhang_SM.pdf
         contacts    = dict(h=2.0, s=20,  w=16,  c=20),   # Number of contacts per person per day, estimated
         dynam_layer = dict(h=0,   s=0,   w=0,   c=0),    # Which layers are dynamic -- none by default
         iso_factor  = dict(h=0.3, s=0.1, w=0.1, c=0.1),  # Multiply beta by this factor for people in isolation
         quar_factor = dict(h=0.6, s=0.2, w=0.2, c=0.2),  # Multiply beta by this factor for people in quarantine
     )
 
+    # Specify defaults for SynthPops -- same as hybrid except for LTCF layer (l)
+    l_pars = dict(beta_layer=1.5,
+                  contacts=10,
+                  dynam_layer=0,
+                  iso_factor=0.2,
+                  quar_factor=0.3
+    )
+    layer_defaults['synthpops'] = sc.dcp(layer_defaults['hybrid'])
+    for key,val in l_pars.items():
+        layer_defaults['synthpops'][key]['l'] = val
+
     # Choose the parameter defaults based on the population type, and get the layer keys
-    if pars['pop_type'] == 'random':
-        defaults = defaults_r
-        default_layer_keys = ['a'] # Although this could be retrieved from the dictionary, make it explicit
-    else:
-        defaults = defaults_h
-        default_layer_keys = ['h', 's', 'w', 'c'] # NB, these must match defaults_h above
+    try:
+        defaults = layer_defaults[pars['pop_type']]
+    except Exception as E:
+        errormsg = f'Cannot load defaults for population type "{pars["pop_type"]}": must be hybrid, random, or synthpops'
+        raise ValueError(errormsg) from E
+    default_layer_keys = list(defaults['beta_layer'].keys()) # All layers should be the same, but use beta_layer for convenience
 
     # Actually set the parameters
     for pkey in layer_pars:
         par = {} # Initialize this parameter
-        default_val = defaults_r[pkey]['a'] # Get the default value for this parameter
+        default_val = layer_defaults['random'][pkey]['a'] # Get the default value for this parameter
 
         # If forcing, we overwrite any existing parameter values
         if force:
@@ -168,42 +193,84 @@ def reset_layer_pars(pars, layer_keys=None, force=False):
     return
 
 
-
-def get_prognoses(by_age=True):
+def get_prognoses(by_age=True, version=None):
     '''
     Return the default parameter values for prognoses
 
     The prognosis probabilities are conditional given the previous disease state.
 
     Args:
-        by_age (bool): whether or not to use age-specific values
+        by_age (bool): whether to use age-specific values (default true)
 
     Returns:
         prog_pars (dict): the dictionary of prognosis probabilities
     '''
 
-    if not by_age:
+    if not by_age: # All rough estimates -- almost always, prognoses by age (below) are used instead
         prognoses = dict(
             age_cutoffs  = np.array([0]),
             symp_probs   = np.array([0.75]),
-            severe_probs = np.array([0.20]),
-            crit_probs   = np.array([0.08]),
-            death_probs  = np.array([0.02]),
+            severe_probs = np.array([0.10]),
+            crit_probs   = np.array([0.04]),
+            death_probs  = np.array([0.01]),
         )
     else:
         prognoses = dict(
             age_cutoffs   = np.array([0,       10,      20,      30,      40,      50,      60,      70,      80,      90,]),     # Age cutoffs (lower limits)
-            sus_ORs       = np.array([0.34,    0.67,    1.00,    1.00,    1.00,    1.00,    1.24,    1.47,    1.47,    1.47]),    # Odds ratios for relative susceptibility -- from https://science.sciencemag.org/content/early/2020/05/04/science.abb8001; 10-20 and 60-70 bins are the average across the ORs
+            sus_ORs       = np.array([0.34,    0.67,    1.00,    1.00,    1.00,    1.00,    1.24,    1.47,    1.47,    1.47]),    # Odds ratios for relative susceptibility -- from Zhang et al., https://science.sciencemag.org/content/early/2020/05/04/science.abb8001; 10-20 and 60-70 bins are the average across the ORs
             trans_ORs     = np.array([1.00,    1.00,    1.00,    1.00,    1.00,    1.00,    1.00,    1.00,    1.00,    1.00]),    # Odds ratios for relative transmissibility -- no evidence of differences
             comorbidities = np.array([1.00,    1.00,    1.00,    1.00,    1.00,    1.00,    1.00,    1.00,    1.00,    1.00]),    # Comorbidities by age -- set to 1 by default since already included in disease progression rates
             symp_probs    = np.array([0.50,    0.55,    0.60,    0.65,    0.70,    0.75,    0.80,    0.85,    0.90,    0.90]),    # Overall probability of developing symptoms (based on https://www.medrxiv.org/content/10.1101/2020.03.24.20043018v1.full.pdf, scaled for overall symptomaticity)
             severe_probs  = np.array([0.00050, 0.00165, 0.00720, 0.02080, 0.03430, 0.07650, 0.13280, 0.20655, 0.24570, 0.24570]), # Overall probability of developing severe symptoms (derived from Table 1 of https://www.imperial.ac.uk/media/imperial-college/medicine/mrc-gida/2020-03-16-COVID19-Report-9.pdf)
             crit_probs    = np.array([0.00003, 0.00008, 0.00036, 0.00104, 0.00216, 0.00933, 0.03639, 0.08923, 0.17420, 0.17420]), # Overall probability of developing critical symptoms (derived from Table 1 of https://www.imperial.ac.uk/media/imperial-college/medicine/mrc-gida/2020-03-16-COVID19-Report-9.pdf)
-            death_probs   = np.array([0.00002, 0.00006, 0.00030, 0.00080, 0.00150, 0.00600, 0.02200, 0.05100, 0.09300, 0.09300]), # Overall probability of dying (https://www.imperial.ac.uk/media/imperial-college/medicine/sph/ide/gida-fellowships/Imperial-College-COVID19-NPI-modelling-16-03-2020.pdf)
+            death_probs   = np.array([0.00002, 0.00002, 0.00010, 0.00032, 0.00098, 0.00265, 0.00766, 0.02439, 0.08292, 0.16190]), # Overall probability of dying -- from O'Driscoll et al., https://www.nature.com/articles/s41586-020-2918-0; last data point from Brazeau et al., https://www.imperial.ac.uk/mrc-global-infectious-disease-analysis/covid-19/report-34-ifr/
         )
+    prognoses = relative_prognoses(prognoses) # Convert to conditional probabilities
 
-    prognoses['death_probs']  /= prognoses['crit_probs']   # Conditional probability of dying, given critical symptoms
-    prognoses['crit_probs']   /= prognoses['severe_probs'] # Conditional probability of symptoms becoming critical, given severe
-    prognoses['severe_probs'] /= prognoses['symp_probs']   # Conditional probability of symptoms becoming severe, given symptomatic
+    # If version is specified, load old parameters
+    if version is not None:
+        version_pars = cvm.get_version_pars(version)
+        version_prognoses = version_pars['prognoses']
+        for key in version_prognoses.keys(): # Only loop over keys that have been populated
+            if key in version_pars: # Only replace keys that exist in the old version
+                prognoses[key] = version_pars[key]
+
+    # Check that lengths match
+    expected_len = len(prognoses['age_cutoffs'])
+    for key,val in prognoses.items():
+        this_len = len(prognoses[key])
+        if this_len != expected_len:
+            errormsg = f'Lengths mismatch in prognoses: {expected_len} age bins specified, but key "{key}" has {this_len} entries'
+            raise ValueError(errormsg)
 
     return prognoses
+
+
+def relative_prognoses(prognoses):
+    '''
+    Convenience function to revert absolute prognoses into relative (conditional)
+    ones. Internally, Covasim uses relative prognoses.
+    '''
+    out = sc.dcp(prognoses)
+    out['death_probs']  /= out['crit_probs']   # Conditional probability of dying, given critical symptoms
+    out['crit_probs']   /= out['severe_probs'] # Conditional probability of symptoms becoming critical, given severe
+    out['severe_probs'] /= out['symp_probs']   # Conditional probability of symptoms becoming severe, given symptomatic
+    return out
+
+
+def absolute_prognoses(prognoses):
+    '''
+    Convenience function to revert relative (conditional) prognoses into absolute
+    ones. Used to convert internally used relative prognoses into more readable
+    absolute ones.
+
+    **Example**::
+
+        sim = cv.Sim()
+        abs_progs = cv.parameters.absolute_prognoses(sim['prognoses'])
+    '''
+    out = sc.dcp(prognoses)
+    out['severe_probs'] *= out['symp_probs']   # Absolute probability of severe symptoms
+    out['crit_probs']   *= out['severe_probs'] # Absolute probability of critical symptoms
+    out['death_probs']  *= out['crit_probs']   # Absolute probability of dying
+    return out
\ No newline at end of file
