@@ -35,6 +35,7 @@ class Sim(cvb.BaseSim):
         popfile  (str):    the filename to load/save the population for this simulation
         load_pop (bool):   whether to load the population from the named file
         save_pop (bool):   whether to save the population to the named file
+        version  (str):    if supplied, use default parameters from this version of Covasim instead of the latest
         kwargs   (dict):   passed to make_pars()
 
     **Examples**::
@@ -43,9 +44,11 @@ class Sim(cvb.BaseSim):
         sim = cv.Sim(pop_size=10e3, datafile='my_data.xlsx')
     '''
 
-    def __init__(self, pars=None, datafile=None, datacols=None, label=None, simfile=None, popfile=None, load_pop=False, save_pop=False, **kwargs):
+    def __init__(self, pars=None, datafile=None, datacols=None, label=None, simfile=None,
+                 popfile=None, load_pop=False, save_pop=False, version=None, **kwargs):
+
         # Create the object
-        default_pars = cvpar.make_pars() # Start with default pars
+        default_pars = cvpar.make_pars(version=version) # Start with default pars
         super().__init__(default_pars) # Initialize and set the parameters as attributes
 
         # Set attributes
@@ -64,14 +67,16 @@ class Sim(cvb.BaseSim):
         self.initialized   = False    # Whether or not initialization is complete
         self.complete      = False    # Whether a simulation has completed running
         self.results_ready = False    # Whether or not results are ready
+        self._default_ver  = version  # Default version of parameters used
         self._orig_pars    = None     # Store original parameters to optionally restore at the end of the simulation
 
         # Now update everything
-        self.set_metadata(simfile, label)  # Set the simulation date and filename
+        self.set_metadata(simfile)  # Set the simulation date and filename
         self.update_pars(pars, **kwargs)   # Update the parameters, if provided
         self.load_data(datafile, datacols) # Load the data, if provided
         if self.load_pop:
-            self.load_population(popfile)      # Load the population, if provided
+            self.load_population(popfile)  # Load the population, if provided
+
         return
 
 
@@ -162,7 +167,7 @@ class Sim(cvb.BaseSim):
         for lp in layer_pars:
             lp_keys = set(self.pars[lp].keys())
             if not lp_keys == set(layer_keys):
-                errormsg = f'At least one layer parameter is inconsistent with the layer keys; all parameters must have the same keys:'
+                errormsg = 'At least one layer parameter is inconsistent with the layer keys; all parameters must have the same keys:'
                 errormsg += f'\nsim.layer_keys() = {layer_keys}'
                 for lp2 in layer_pars: # Fail on first error, but re-loop to list all of them
                     errormsg += f'\n{lp2} = ' + ', '.join(self.pars[lp2].keys())
@@ -198,14 +203,14 @@ class Sim(cvb.BaseSim):
         start_day = self['start_day'] # Shorten
         if start_day in [None, 0]: # Use default start day
             start_day = '2020-03-01'
-        self['start_day'] = cvm.date(start_day)
+        self['start_day'] = sc.date(start_day)
 
         # Handle end day and n_days
         end_day = self['end_day']
         n_days = self['n_days']
         if end_day:
-            self['end_day'] = cvm.date(end_day)
-            n_days = cvm.daydiff(self['start_day'], self['end_day'])
+            self['end_day'] = sc.date(end_day)
+            n_days = sc.daydiff(self['start_day'], self['end_day'])
             if n_days <= 0:
                 errormsg = f"Number of days must be >0, but you supplied start={str(self['start_day'])} and end={str(self['end_day'])}, which gives n_days={n_days}"
                 raise ValueError(errormsg)
@@ -238,6 +243,13 @@ class Sim(cvb.BaseSim):
         if validate_layers:
             self.validate_layer_pars()
 
+        # Handle verbose
+        if self['verbose'] == 'brief':
+            self['verbose'] = -1
+        if not sc.isnumber(self['verbose']):
+            errormsg = f'Verbose argument should be either "brief", -1, or a float, not {type(self["verbose"])} "{self["verbose"]}"'
+            raise ValueError(errormsg)
+
         return
 
 
@@ -361,7 +373,7 @@ class Sim(cvb.BaseSim):
         # Handle inputs
         if verbose is None:
             verbose = self['verbose']
-        if verbose:
+        if verbose>0:
             print(f'Initializing sim with {self["pop_size"]:0n} people for {self["n_days"]} days')
         if load_pop and self.popdict is None:
             self.load_population(popfile=popfile)
@@ -540,7 +552,7 @@ class Sim(cvb.BaseSim):
         return
 
 
-    def run(self, do_plot=False, until=None, restore_pars=True, reset_seed=True, verbose=None, **kwargs):
+    def run(self, do_plot=False, until=None, restore_pars=True, reset_seed=True, verbose=None, output=False, **kwargs):
         '''
         Run the simulation.
 
@@ -549,7 +561,8 @@ class Sim(cvb.BaseSim):
             until (int/str): day or date to run until
             restore_pars (bool): whether to make a copy of the parameters before the run and restore it after, so runs are repeatable
             reset_seed (bool): whether to reset the random number stream immediately before run
-            verbose (float): level of detail to print, e.g. 0 = no output, 0.2 = print every 5th day, 1 = print every day
+            verbose (float): level of detail to print, e.g. -1 = one-line output, 0 = no output, 0.1 = print every 10th day, 1 = print every day
+            output (bool): whether to return the results dictionary as output
             kwargs (dict): passed to sim.plot()
 
         Returns:
@@ -559,13 +572,13 @@ class Sim(cvb.BaseSim):
         # Initialization steps -- start the timer, initialize the sim and the seed, and check that the sim hasn't been run
         T = sc.tic()
 
-        if verbose is None:
-            verbose = self['verbose']
-
         if not self.initialized:
             self.initialize()
             self._orig_pars = sc.dcp(self.pars) # Create a copy of the parameters, to restore after the run, in case they are dynamically modified
 
+        if verbose is None:
+            verbose = self['verbose']
+
         if reset_seed:
             # Reset the RNG. If the simulation is newly created, then the RNG will be reset by sim.initialize() so the use case
             # for resetting the seed here is if the simulation has been partially run, and changing the seed is required
@@ -599,7 +612,7 @@ class Sim(cvb.BaseSim):
                 string = f'  Running {simlabel}{self.datevec[self.t]} ({self.t:2.0f}/{self.pars["n_days"]}) ({elapsed:0.2f} s) '
                 if verbose >= 2:
                     sc.heading(string)
-                else:
+                elif verbose>0:
                     if not (self.t % int(1.0/verbose)):
                         sc.progressbar(self.t+1, self.npts, label=string, length=20, newline=True)
 
@@ -612,7 +625,10 @@ class Sim(cvb.BaseSim):
             sc.printv(f'Run finished after {elapsed:0.2f} s.\n', 1, verbose)
             if do_plot: # Optionally plot
                 self.plot(**kwargs)
-            return self.results
+            if output:
+                return self.results
+            else:
+                return
         else:
             return # If not complete, return nothing
 
@@ -650,6 +666,13 @@ class Sim(cvb.BaseSim):
                 if key not in preserved:
                     self.pars[key] = self._orig_pars.pop(key) # Restore everything except for the analyzers and interventions
 
+        # Optionally print summary output
+        if verbose: # Verbose is any non-zero value
+            if verbose>0: # Verbose is any positive number
+                self.summarize() # Print medium-length summary of the sim
+            else:
+                self.brief() # Print brief summary of the sim
+
         return
 
 
@@ -659,7 +682,7 @@ class Sim(cvb.BaseSim):
         self.compute_yield()
         self.compute_doubling()
         self.compute_r_eff()
-        self.summarize(verbose=verbose, update=True)
+        self.compute_summary()
         return
 
 
@@ -811,7 +834,7 @@ class Sim(cvb.BaseSim):
 
         # Method not recognized
         else:
-            errormsg = f'Method must be "daily", "infected", or "outcome", not "{method}"'
+            errormsg = f'Method must be "daily", "infectious", or "outcome", not "{method}"'
             raise ValueError(errormsg)
 
         # Set the values and return
@@ -855,69 +878,108 @@ class Sim(cvb.BaseSim):
         return self.results['gen_time']
 
 
-    def summarize(self, full=False, t=None, verbose=None, output=False, update=True):
+    def compute_summary(self, full=None, t=None, update=True, output=False):
         '''
-        Print a summary of the simulation, drawing from the last time point in the simulation.
+        Compute the summary dict and string for the sim. Used internally; see
+        sim.summarize() for the user version.
 
         Args:
             full (bool): whether or not to print all results (by default, only cumulative)
             t (int/str): day or date to compute summary for (by default, the last point)
-            verbose (bool): whether to print to screen (default: same as sim)
+            update (bool): whether to update the stored sim.summary
             output (bool): whether to return the summary
-            update (bool): whether to update the summary stored in the sim (sim.summary)
         '''
-        if self.results_ready:
+        if t is None:
+            t = self.day(self.t)
 
-            if t is None:
-                t = self.day(self.t)
+        # Compute the summary
+        summary = sc.objdict()
+        for key in self.result_keys():
+            summary[key] = self.results[key][t]
 
-            if verbose is None:
-                verbose = self['verbose']
+        # Update the stored state
+        if update:
+            self.summary = summary
 
-            summary = sc.objdict()
-            for key in self.result_keys():
-                summary[key] = self.results[key][t]
-
-            summary_str = 'Simulation summary:\n'
-            for key in self.result_keys():
-                if full or key.startswith('cum_'):
-                    summary_str += f'   {summary[key]:5.0f} {self.results[key].name.lower()}\n'
-
-            if verbose:
-                print(summary_str)
-            if update:
-                self.summary = summary
-            if output:
-                return summary
+        # Optionally return
+        if output:
+            return summary
         else:
-            return self.brief(output=output) # If the simulation hasn't been run, default to the brief summary
+            return
+
+
+    def summarize(self, full=False, t=None, output=False):
+        '''
+        Print a medium-length summary of the simulation, drawing from the last time
+        point in the simulation by default. Called by default at the end of a sim run.
+        See also sim.disp() (detailed output) and sim.brief() (short output).
+
+        Args:
+            full (bool): whether or not to print all results (by default, only cumulative)
+            t (int/str): day or date to compute summary for (by default, the last point)
+            output (bool): whether to return the summary instead of printing it
+
+        **Examples**::
+
+            sim = cv.Sim(label='Example sim', verbose=0) # Set to run silently
+            sim.run() # Run the sim
+            sim.summarize() # Print medium-length summary of the sim
+            sim.summarize(t=24, full=True) # Print a "slice" of all sim results on day 24
+        '''
+        # Compute the summary
+        summary = self.compute_summary(full=full, t=t, update=False, output=True)
+
+        # Construct the output string
+        labelstr = f' "{self.label}"' if self.label else ''
+        string = f'Simulation{labelstr} summary:\n'
+        for key in self.result_keys():
+            if full or key.startswith('cum_'):
+                string += f'   {summary[key]:5.0f} {self.results[key].name.lower()}\n'
+
+        # Print or return string
+        if not output:
+            print(string)
+        else:
+            return string
+
+
+    def disp(self, output=False):
+        '''
+        Display a verbose description of a sim. See also sim.summarize() (medium
+        length output) and sim.brief() (short output).
+
+        Args:
+            output (bool): if true, return a string instead of printing output
+
+        **Example**::
+
+            sim = cv.Sim(label='Example sim', verbose=0) # Set to run silently
+            sim.run() # Run the sim
+            sim.disp() # Displays detailed output
+        '''
+        string = self._disp()
+        if not output:
+            print(string)
+        else:
+            return string
 
 
     def brief(self, output=False):
-        ''' Return a one-line description of a sim '''
+        '''
+        Print a one-line description of a sim. See also sim.disp() (detailed output)
+        and sim.summarize() (medium length output). The symbol "⚙" is used to show
+        infections, and "☠" is used to show deaths.
 
-        if self.results_ready:
-            infections = self.summary['cum_infections']
-            deaths = self.summary['cum_deaths']
-            results = f'{infections:n}⚙, {deaths:n}☠'
-        else:
-            results = 'not run'
+        Args:
+            output (bool): if true, return a string instead of printing output
 
-        if self.label:
-            label = f'"{self.label}"'
-        else:
-            label = '<no label>'
-
-        start = cvm.date(self['start_day'], as_date=False)
-        if self['end_day']:
-            end = cvm.date(self['end_day'], as_date=False)
-        else:
-            end = cvm.date(self['n_days'], start_date=start)
-
-        pop_size = self['pop_size']
-        pop_type = self['pop_type']
-        string   = f'Sim({label}; {start}—{end}; pop: {pop_size:n} {pop_type}; epi: {results})'
+        **Example**::
 
+            sim = cv.Sim(label='Example sim', verbose=0) # Set to run silently
+            sim.run() # Run the sim
+            sim.brief() # Prints one-line output
+        '''
+        string = self._brief()
         if not output:
             print(string)
         else:
@@ -1029,6 +1091,7 @@ class Sim(cvb.BaseSim):
             colors       (dict): Custom color for each result, must be a dictionary with one entry per result key in to_plot
             sep_figs     (bool): Whether to show separate figures for different results instead of subplots
             fig          (fig):  Handle of existing figure to plot into
+            ax           (axes): Axes instance to plot into
 
         Returns:
             fig: Figure handle
