@@ -15,7 +15,6 @@ import numpy as np
 from covasim import Sim, parameters, change_beta, test_prob, contact_tracing, sequence
 
 
-
 class TestProperties:
     class ParameterKeys:
         class SimulationKeys:
@@ -41,7 +40,7 @@ class TestProperties:
             asymptomatic_transmission_multiplier = 'asym_factor'
             diagnosis_transmission_factor = 'iso_factor'
             contact_transmission_factor = 'cont_factor'
-            contacts_per_agent = 'contacts'  # TODO: need to construct a population now
+            contacts_per_agent = 'contacts'
             beta_population_specific = 'beta_pop'
             contacts_population_specific = 'contacts_pop'
             pass
@@ -299,7 +298,7 @@ class CovaSimTest(unittest.TestCase):
                                        asymp_prob=asymptomatic_prob,
                                        asymp_quar_prob=asymptomatic_quarantine_prob,
                                        symp_quar_prob=symp_quar_prob,
-                                       test_sensitivity=test_sensitivity,
+                                       sensitivity=test_sensitivity,
                                        loss_prob=loss_prob,
                                        test_delay=test_delay,
                                        start_day=start_day)
@@ -309,10 +308,7 @@ class CovaSimTest(unittest.TestCase):
                                          start_day,
                                          trace_probabilities=None,
                                          trace_times=None):
-        #  see ../tests/test_interventions_testing.test_tracedelay
-        #  trace_probs = {'h': 1, 's': 1, 'w': 1, 'c': 1}
-        #  trace_time = {'h': 0, 's': 1, 'w': 1, 'c': 2}
-        #  pars.quar_period = 60 # 60 days
+
         if not trace_probabilities:
             trace_probabilities = {'h': 1, 's': 1, 'w': 1, 'c': 1}
             pass
@@ -485,25 +481,6 @@ class TestSupportTests(CovaSimTest):
         self.assertTrue(json_file_found, msg=f"Expected {self.expected_result_filename} to be found.")
     pass
 
-    @unittest.skip("Cannot easily specify number contacts to be small, need to construct a population")
-    def test_run_microsim(self):
-        """
-        Runs a super short simulation
-        Verifies that the microsim parameters were created and honored
-        """
-        self.assertIsNone(self.simulation_parameters)
-        self.assertIsNone(self.sim)
-        self.set_microsim()
-        self.run_sim()
-        result_data = self.simulation_result["results"]
-        resultKeys = TestProperties.ResultsDataKeys
-        microsimParams = TestProperties.SpecializedSimulations.Microsim
-        self.assertEqual(len(result_data[resultKeys.recovered_at_timestep]),
-                         microsimParams.n + 1)
-        self.assertEqual(result_data[resultKeys.exposed_at_timestep][0],
-                         microsimParams.pop_infected)
-        pass
-
     def test_everyone_infected(self):
         """
         All agents start infected
